package ModifiedSchedules;

import InitialSchedules.*;
import ScheduleUtil.*;
import jdistlib.rng.RandomEngine;

public class ModifiedSchedulesArray extends SchedulesArray{

    private final SchedulesArray initialSchedulesArray;
    public int numberOfRecoveringActionsToPerform;
    
    public ModifiedSchedulesArray(SchedulesArray initialSchedulesArray, ParametersSchedules myModifiedScheduleParameters, RandomEngine randAbsence) {
        super(initialSchedulesArray);
        this.myScheduleParameters = myModifiedScheduleParameters;
        this.initialSchedulesArray = initialSchedulesArray;
        generateAbsenceSimulation(randAbsence);
        calculateNumberOfRecoveringActionsToPerform();
        this.totalScheduleCost = EmployeeCostCalculator.getFullModifiedScheduleCost(this, myScheduleParameters);
    } 

    private void generateAbsenceSimulation(RandomEngine randAbsence) {
        
        for (int i = 0; i < getNumberPartTimeEmployee(); i++) {
            boolean[] a = new AbsencesVector(myScheduleParameters.getWorkPeriodsPerSchedule(), randAbsence).getAbsencesVector();
            for (int j = 0; j < myScheduleParameters.getWorkPeriodsPerSchedule(); j++) {
                this.partTimeSchedules[i][j] = this.initialSchedulesArray.isPartTimeEmployeeWorking(i, j) && a[j];
            }
        }

        for (int i = 0; i < getNumberFullTimeEmployee(); i++) {
            boolean[] a = new AbsencesVector(myScheduleParameters.getWorkPeriodsPerSchedule(), randAbsence).getAbsencesVector();
            for (int j = 0; j < myScheduleParameters.getWorkPeriodsPerSchedule(); j++) {
                this.fullTimeSchedules[i][j] = this.initialSchedulesArray.isFullTimeEmployeeWorking(i, j) && a[j];
            }
        }

    }
    
    public int getAbsencePeriodsPerPartTimeEmployees(int employee) {
        int absencePeriodsPerPartTimeEmployees = 0;
        for (int workPeriod = 0; workPeriod < myScheduleParameters.getWorkPeriodsPerSchedule(); workPeriod++) {
            if (this.isPartTimeEmployeeAbsent(employee, workPeriod)) {absencePeriodsPerPartTimeEmployees += 1;}
        }
        return absencePeriodsPerPartTimeEmployees;
    }
    
    public int getAbsencePeriodsPerFullTimeEmployees(int employee) {
        int absencePeriodsPerFullTimeEmployees = 0;
        for (int workPeriod = 0; workPeriod < myScheduleParameters.getWorkPeriodsPerSchedule(); workPeriod++) {
            if (this.isFullTimeEmployeeAbsent(employee, workPeriod)) {absencePeriodsPerFullTimeEmployees += 1;}
        }
        return absencePeriodsPerFullTimeEmployees;
    }

    public boolean isPartTimeEmployeeAbsent(int employee, int workPeriod) {
        return this.partTimeSchedules[employee][workPeriod] != this.initialSchedulesArray.isPartTimeEmployeeWorking(employee, workPeriod);
    }
    
    public boolean isFullTimeEmployeeAbsent(int employee, int workPeriod) {
        return this.fullTimeSchedules[employee][workPeriod] != this.initialSchedulesArray.isFullTimeEmployeeWorking(employee, workPeriod);
    }
    
    private void calculateNumberOfRecoveringActionsToPerform() {
        this.numberOfRecoveringActionsToPerform = 0;
        for (int workPeriod = 0; workPeriod < myScheduleParameters.getWorkPeriodsPerSchedule(); workPeriod++) {
            this.numberOfRecoveringActionsToPerform += this.myScheduleParameters.getRequiredWorkForce(workPeriod) - this.getEmployeesWorkingPerWorkPeriod(workPeriod);
        }
    }
    
}