/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package ScheduleUtil;

import ModifiedSchedules.ModifiedSchedulesArray;
import InitialSchedules.ParametersSchedules;
import SchedulesRecovery.RecoveredSchedulesArray;

/**
 *
 * @author frabe
 */
public class EmployeeCostCalculator {
    
    public static int getFullInitialScheduleCost( SchedulesArray myInitialSchedule, ParametersSchedules myScheduleParameters) {
        int totalCost = 0;
        for (int partTimeEmployee = 0 ; partTimeEmployee < myInitialSchedule.getNumberPartTimeEmployee() ; partTimeEmployee++) {
            int numberWorkingPeriod = myInitialSchedule.getWorkingPeriodsPerPartTimeEmployees(partTimeEmployee);
            totalCost += myScheduleParameters.getFixedCostOfPartTimeEmployeesPerSchedule() + 
                    numberWorkingPeriod * myScheduleParameters.getWorkingPeriodCostOfPartTimeEmployeesPaidAtRegularHourlyRate();
        }
        for (int fullTimeEmployee = 0 ; fullTimeEmployee < myInitialSchedule.getNumberFullTimeEmployee() ; fullTimeEmployee++) {
            int numberWorkingPeriod = myInitialSchedule.getWorkingPeriodsPerFullTimeEmployees(fullTimeEmployee);
            totalCost += myScheduleParameters.getFixedCostOfFullTimeEmployeesPerSchedule() + 
                    numberWorkingPeriod * myScheduleParameters.getWorkingPeriodCostOfFullTimeEmployeesPaidAtRegularHourlyRate();
        }
        return totalCost;
    }
    
    public static int getFullModifiedScheduleCost( ModifiedSchedulesArray myModifiedSchedule, ParametersSchedules myScheduleParameters) {
        int totalCost = 0;
        for (int partTimeEmployee = 0 ; partTimeEmployee < myModifiedSchedule.getNumberPartTimeEmployee() ; partTimeEmployee++) {
            int numberWorkingPeriod = myModifiedSchedule.getWorkingPeriodsPerPartTimeEmployees(partTimeEmployee);
            totalCost += myScheduleParameters.getFixedCostOfPartTimeEmployeesPerSchedule() + 
                    numberWorkingPeriod * myScheduleParameters.getWorkingPeriodCostOfPartTimeEmployeesPaidAtRegularHourlyRate();
        }
        for (int fullTimeEmployee = 0 ; fullTimeEmployee < myModifiedSchedule.getNumberFullTimeEmployee() ; fullTimeEmployee++) {
            int numberWorkingPeriod = myModifiedSchedule.getWorkingPeriodsPerFullTimeEmployees(fullTimeEmployee);
            int numberAbsencePeriod = myModifiedSchedule.getAbsencePeriodsPerFullTimeEmployees(fullTimeEmployee);
            totalCost += myScheduleParameters.getFixedCostOfFullTimeEmployeesPerSchedule() + 
                    (numberWorkingPeriod + numberAbsencePeriod) * myScheduleParameters.getWorkingPeriodCostOfFullTimeEmployeesPaidAtRegularHourlyRate();
        }
        return totalCost;
    }
    
    public static int getFullRecoveredScheduleCost( RecoveredSchedulesArray myRecoveredSchedule, ParametersSchedules myScheduleParameters) {
        int totalCost = 0;
        
        for (int partTimeEmployee = 0 ; partTimeEmployee < myRecoveredSchedule.getNumberPartTimeEmployee() ; partTimeEmployee++) {
            int numberWorkingPeriod = myRecoveredSchedule.getWorkingPeriodsPerPartTimeEmployees(partTimeEmployee);
            totalCost += myScheduleParameters.getFixedCostOfPartTimeEmployeesPerSchedule() + 
                    numberWorkingPeriod * myScheduleParameters.getWorkingPeriodCostOfPartTimeEmployeesPaidAtRegularHourlyRate();
            
        }
        
        ModifiedSchedulesArray myModifiedSchedule = myRecoveredSchedule.modifiedSchedulesArray;
        for (int fullTimeEmployee = 0 ; fullTimeEmployee < myRecoveredSchedule.getNumberFullTimeEmployee() ; fullTimeEmployee++) {
            int numberWorkingPeriod = myRecoveredSchedule.getWorkingPeriodsPerFullTimeEmployees(fullTimeEmployee);
            int numberAbsencePeriod = myModifiedSchedule.getAbsencePeriodsPerFullTimeEmployees(fullTimeEmployee);
            totalCost += myScheduleParameters.getFixedCostOfFullTimeEmployeesPerSchedule();
            int regularHourlyRateCost; int overtimeHourlyRateCost;
            if (numberWorkingPeriod + numberAbsencePeriod <= myScheduleParameters.getWorkingPeriodsPaidAtRegularHourlyRatePerSchedule() ) {
                regularHourlyRateCost = (numberWorkingPeriod + numberAbsencePeriod) * myScheduleParameters.getWorkingPeriodCostOfFullTimeEmployeesPaidAtRegularHourlyRate();
                overtimeHourlyRateCost = 0;
                
            } else {
                regularHourlyRateCost =  myScheduleParameters.getWorkingPeriodsPaidAtRegularHourlyRatePerSchedule() * 
                                         myScheduleParameters.getWorkingPeriodCostOfFullTimeEmployeesPaidAtRegularHourlyRate();
                overtimeHourlyRateCost = (numberWorkingPeriod + numberAbsencePeriod - myScheduleParameters.getWorkingPeriodsPaidAtRegularHourlyRatePerSchedule()) *
                                         myScheduleParameters.getWorkingPeriodCostOfFullTimeEmployeesPaidAtOvertimeHourlyRate();
                
            }
            
            totalCost += regularHourlyRateCost + overtimeHourlyRateCost;
        }

        return totalCost;
    }

    public static int getPartTimeEmployeeCost(boolean[][] currentEmployeesSchedule, int employee, ParametersSchedules myScheduleParameters) {
        int numberWorkingPeriod = 0;
        for (int workPeriod = 0 ; workPeriod < currentEmployeesSchedule.length ; workPeriod++) {
            if (currentEmployeesSchedule[employee][workPeriod]) {
                numberWorkingPeriod++;
            }
        }
        return myScheduleParameters.getFixedCostOfPartTimeEmployeesPerSchedule() +
                numberWorkingPeriod * myScheduleParameters.getWorkingPeriodCostOfPartTimeEmployeesPaidAtRegularHourlyRate();
    }
    
    public static int getFullTimeEmployeeCost(boolean[][] currentEmployeesSchedule, ModifiedSchedulesArray myModifiedSchedule, int employee, ParametersSchedules myScheduleParameters) {
        int numberWorkingPeriod = 0;
        for (int workPeriod = 0 ; workPeriod < currentEmployeesSchedule.length ; workPeriod++) {
            if (currentEmployeesSchedule[employee][workPeriod]) {
                numberWorkingPeriod++;
            }
        }
        int numberAbsencePeriod = myModifiedSchedule.getAbsencePeriodsPerFullTimeEmployees(employee);
        int regularHourlyRateCost; int overtimeHourlyRateCost;
        if (numberWorkingPeriod + numberAbsencePeriod <= myScheduleParameters.getWorkingPeriodsPaidAtRegularHourlyRatePerSchedule() ) {
            regularHourlyRateCost = (numberWorkingPeriod + numberAbsencePeriod) * myScheduleParameters.getWorkingPeriodCostOfFullTimeEmployeesPaidAtRegularHourlyRate();
            overtimeHourlyRateCost = 0;
        } else {
            regularHourlyRateCost =  myScheduleParameters.getWorkingPeriodsPaidAtRegularHourlyRatePerSchedule() * 
                                     myScheduleParameters.getWorkingPeriodCostOfFullTimeEmployeesPaidAtRegularHourlyRate();
            overtimeHourlyRateCost = (numberWorkingPeriod + numberAbsencePeriod - myScheduleParameters.getWorkingPeriodsPaidAtRegularHourlyRatePerSchedule()) *
                                     myScheduleParameters.getWorkingPeriodCostOfFullTimeEmployeesPaidAtOvertimeHourlyRate();
        }
        return myScheduleParameters.getFixedCostOfFullTimeEmployeesPerSchedule() + regularHourlyRateCost + overtimeHourlyRateCost;
    }
}
